@@ -1,4 +1,4 @@
-__version__ = "3.1.7"
+__version__ = "3.8.0"
 
 import pathlib  # isort:skip
 
@@ -7,11 +7,9 @@ APP_NAME = "SPFY"
 
 import os  # isort:skip
 
-ENV = os.getenv(f"{APP_NAME.upper()}_ENV", "config")  # isort:skip
-
 import kick  # isort:skip
 
-kick.start(APP_NAME.lower(), config_variant=ENV)  # isort:skip
+kick.start(APP_NAME.lower())  # isort:skip
 
 from kick import config, logger  # isort:skip
 
@@ -22,7 +20,7 @@ if config.unsplash.auth.client_id:
     auth = Auth(**config.unsplash.auth)
     Unsplash = Api(auth)
 from .client import SpotifyClient
-from .result import SpotifyResult
-from .wrapper import Spotify
 from .constants import *
 from .exceptions import *
+from .result import SpotifyResult
+from .wrapper import Spotify
