@@ -1,17 +1,17 @@
 #!/usr/bin/env python3
 import fire
-import kick
-from pony.orm import db_session
 
-from .. import APP_NAME, config, logger
-from .client import SpotifyClient
+from .. import config, logger
+from ..constants import AuthFlow
 from ..mixins.asynch import PlayerMixin, RecommenderMixin
+from .client import SpotifyClient
 
 # pylint: disable=too-many-ancestors
 
 
 class Spotify(SpotifyClient, PlayerMixin, RecommenderMixin):
     """Spotify high-level wrapper."""
+
     cli = False
     loop = None
 
@@ -25,18 +25,19 @@ class Spotify(SpotifyClient, PlayerMixin, RecommenderMixin):
         names = [name for name in names if not name.startswith("_") and name != "user"]
         return names
 
-    async def auth(self, email=config.auth.email, username=config.auth.username):
+    async def auth(
+        self, email=config.auth.email, username=config.auth.username, server=False
+    ):
         if self.cli and not self.is_authenticated:
-            try:
-                await self.authenticate(email=email, username=username)
-            except Exception as exc:
-                logger.exception(exc)
+            if server:
+                await self.authenticate(flow=AuthFlow.CLIENT_CREDENTIALS)
+            else:
+                try:
+                    await self.authenticate(email=email, username=username)
+                except Exception as exc:
+                    logger.exception(exc)
         return self
 
-    @staticmethod
-    def update_config(name="config"):
-        kick.update_config(APP_NAME.lower(), variant=name)
-
 
 def main():
     """Main function."""
@@ -53,8 +54,7 @@ def main():
         Spotify.cli = True
         Spotify.loop = asyncio.get_event_loop()
         spotify = Spotify()
-        with db_session:
-            fire.Fire(spotify)
+        fire.Fire(spotify)
     except KeyboardInterrupt:
         print("Quitting")
     finally:
