@@ -1,41 +1,45 @@
-import os
-import re
-import time
-import random
+# pylint: disable=too-many-lines
 import asyncio
-from io import BytesIO
-from enum import IntEnum
-from uuid import UUID, NAMESPACE_URL, uuid4, uuid5
-from datetime import date, datetime
+import os
+import random
+import re
+import string
+import time
 from collections import OrderedDict
+from datetime import date, datetime
+from enum import IntEnum
+from io import BytesIO
+from uuid import NAMESPACE_URL, UUID, uuid4, uuid5
 
 import aiohttp
-import requests
 import psycopg2.extras
+import requests
+from colorthief import ColorThief
 from first import first
 from pony.orm import (
-    Set,
-    Json,
     Database,
-    Optional,
-    Required,
-    PrimaryKey,
+    Json,
     ObjectNotFound,
-    desc,
-    select,
-    ormtypes,
-    sql_debug,
+    Optional,
+    PrimaryKey,
+    Required,
+    Set,
     db_session,
+    desc,
+    ormtypes,
+    select,
+    sql_debug,
 )
-from pycountry import countries
-from colorthief import ColorThief
 from pony.orm.core import CacheIndexError
-from unsplash.errors import UnsplashError
 from psycopg2.extensions import register_adapter
+from pycountry import countries
+from unsplash.errors import UnsplashConnectionError, UnsplashError
 
 from .. import Unsplash, config, logger
 from ..constants import TimeRange
+from ..sql import SQL, SQL_DEFAULT
 
+DIGITS_PATTERN = re.compile(r"[0-9]")
 register_adapter(ormtypes.TrackedDict, psycopg2.extras.Json)
 if os.getenv("SQL_DEBUG"):
     sql_debug(True)
@@ -45,40 +49,189 @@ if os.getenv("SQL_DEBUG"):
 db = Database()
 
 
+def create_condition(op="AND", firstsub=1, **fields):
+    condition = f" {op} ".join(
+        f"{field} = ${i + firstsub}" for i, field in enumerate(fields.keys())
+    )
+    return condition
+
+
+def format_param(param):
+    if isinstance(param, (int, float, bool)):
+        return str(param)
+    if isinstance(param, (str, bytes, dict, date, datetime, UUID)):
+        tag = "".join(random.sample(string.ascii_letters, 3))
+        return f"${tag}${param}${tag}$"
+    if param is None:
+        return "null"
+    return param
+
+
 class ImageMixin:
+    @classmethod
+    async def image_pg(cls, conn, width=None, height=None, **fields):
+        condition = create_condition(firstsub=2, **fields)
+        if width:
+            image = await conn.fetchrow(
+                f"SELECT * FROM images WHERE width >= $1 AND {condition} ORDER BY width LIMIT 1",
+                width,
+                *fields.values(),
+            )
+        elif height:
+            image = await conn.fetchrow(
+                f"SELECT * FROM images WHERE height >= $1 AND {condition} ORDER BY height LIMIT 1",
+                height,
+                *fields.values(),
+            )
+        else:
+            image = await conn.fetchrow(
+                f"SELECT * FROM images WHERE {condition} ORDER BY width DESC LIMIT 1",
+                *fields.values(),
+            )
+        return image
 
     def image(self, width=None, height=None):
         if width:
-            image = self.images.select().where(lambda i: i.width >= width).order_by(
-                Image.width
-            ).first()
+            image = (
+                self.images.select()
+                .where(lambda i: i.width >= width)
+                .order_by(Image.width)
+                .first()
+            )
         elif height:
-            image = self.images.select().where(lambda i: i.height >= height).order_by(
-                Image.height
-            ).first()
+            image = (
+                self.images.select()
+                .where(lambda i: i.height >= height)
+                .order_by(Image.height)
+                .first()
+            )
         else:
             image = self.images.select().order_by(desc(Image.width)).first()
         return image
 
+    @classmethod
+    def get_image_queries_pg(cls, key):
+        words = key.split()
+        stems = [[w[:i] for i in range(len(w), 2, -1)] for w in words]
+        queries = [key, *words, *sum(stems, [])]
+        return queries + [f"{query} music" for query in queries]
+
     def get_image_queries(self):
         words = self.name.split()
         stems = [[w[:i] for i in range(len(w), 2, -1)] for w in words]
         queries = [*words, self.name, *sum(stems, [])]
         return [f"{query} music" for query in queries]
 
-    async def fetch_unsplash_image(self, width=None, height=None):
-        image = self.image(width, height)
-        if image:
-            return image
+    @classmethod
+    async def get_image_fields(cls, image_key=None, **fields):
+        logger.info("Getting image for key=%s %s", image_key, fields)
 
-        queries = self.get_image_queries()
+        key = image_key or " ".join(str(v) for v in fields.values())
+        queries = cls.get_image_queries_pg(key)
+
+        photo = await cls.get_unsplash_photo(queries)
+        if photo is None:
+            return None
+
+        ratio = photo.height / photo.width
+        params = {
+            "color": photo.color,
+            "unsplash_id": photo.id,
+            "unsplash_user_fullname": photo.user.name,
+            "unsplash_user_username": photo.user.username,
+            **fields,
+        }
+        image_fields = [
+            {
+                **params,
+                "url": photo.urls.full,
+                "width": photo.width,
+                "height": photo.height,
+            },
+            {
+                **params,
+                "url": photo.urls.regular,
+                "width": Image.REGULAR,
+                "height": int(round(ratio * Image.REGULAR)),
+            },
+            {
+                **params,
+                "url": photo.urls.small,
+                "width": Image.SMALL,
+                "height": int(round(ratio * Image.SMALL)),
+            },
+            {
+                **params,
+                "url": photo.urls.thumb,
+                "width": Image.THUMB,
+                "height": int(round(ratio * Image.THUMB)),
+            },
+        ]
+        image_fields = [
+            OrderedDict(sorted(d.items(), key=lambda t: t[0])) for d in image_fields
+        ]
+
+        return image_fields, fields
+
+    @classmethod
+    async def upsert_unsplash_image(cls, conn, image_fields, **updated_fields):
+        logger.debug(
+            "Upserting image: %s | UPDATED: [%s]", image_fields, updated_fields
+        )
+
+        columns = ", ".join(image_fields[0].keys())
+        values = ",\n".join(
+            f"({', '.join([format_param(p) for p in im.values()])})"
+            for im in image_fields
+        )
+        updated_fields_str = ", ".join(
+            f"{col} = ${i+1}" for i, col in enumerate(updated_fields.keys())
+        )
+        images = await conn.fetch(
+            f"""INSERT INTO images AS im ({columns})
+            VALUES {values}
+            ON CONFLICT (url) DO UPDATE SET {updated_fields_str}
+            RETURNING *
+            """,
+            *updated_fields.values(),
+        )
+        return [dict(i) for i in images]
+
+    @classmethod
+    async def fetch_unsplash_image_pg(
+        cls, conn, width=None, height=None, image_key=None, **fields
+    ):
+        image_fields, updated_fields = await cls.get_image_fields(image_key, **fields)
+        images = await cls.upsert_unsplash_image(conn, image_fields, **updated_fields)
+
+        if not images:
+            return None
+        return cls.get_optimal_image(images, width=width, height=height)
+
+    @staticmethod
+    def get_optimal_image(images, width=None, height=None):
+        images = sorted(images, key=lambda i: i.get("width" or 0))
+        if width:
+            return first(
+                images, key=lambda i: (i.get("width") or 0) >= width, default=images[0]
+            )
+        if height:
+            return first(
+                images,
+                key=lambda i: (i.get("height") or 0) >= height,
+                default=images[0],
+            )
+        return images[0]
+
+    @staticmethod
+    async def get_unsplash_photo(queries):
         photo = None
         for query in queries:
             try:
                 photos = await Unsplash.photo.random(
                     query=query, orientation="squarish"
                 )
-            except UnsplashError:
+            except (UnsplashError, UnsplashConnectionError):
                 continue
 
             if photos:
@@ -86,11 +239,21 @@ class ImageMixin:
                 break
 
         else:
-            photo = (
-                await Unsplash.photo.random(query="music", orientation="squarish")
-            )[
-                0
-            ]
+            try:
+                photo = (
+                    await Unsplash.photo.random(query="music", orientation="squarish")
+                )[0]
+            except (UnsplashError, UnsplashConnectionError):
+                photo = None
+        return photo
+
+    async def fetch_unsplash_image(self, width=None, height=None):
+        image = self.image(width, height)
+        if image:
+            return image
+
+        queries = self.get_image_queries()
+        photo = await self.get_unsplash_photo(queries)
         if photo is None:
             return None
 
@@ -104,7 +267,10 @@ class ImageMixin:
         params = {self.__class__.__name__.lower(): self, "unsplash_id": photo.id}
         image_exists = False
         for url in (
-            photo.urls.full, photo.urls.regular, photo.urls.small, photo.urls.thumb
+            photo.urls.full,
+            photo.urls.regular,
+            photo.urls.small,
+            photo.urls.thumb,
         ):
             image = Image.get(url=url)
             if image:
@@ -148,19 +314,25 @@ class User(db.Entity, ImageMixin):
     DEFAULT_EMAIL = "spfy@backend"
     DEFAULT_USERNAME = "spfy-backend"
     DEFAULT_USERID = uuid5(NAMESPACE_URL, DEFAULT_USERNAME)
-    id = PrimaryKey(UUID, default=uuid4)  # pylint: disable=redefined-builtin
-    email = Optional(str, unique=True, index=True)
+    id = PrimaryKey(
+        UUID, default=uuid4, sql_default=SQL_DEFAULT.uuid4
+    )  # pylint: disable=redefined-builtin
+    email = Optional(str, unique=True, index=True, sql_default="''")
     username = Required(str, unique=True, index=True)
     country = Required("Country")
     preferred_country = Optional("Country")
     spotify_user = Optional("SpotifyUser")
-    display_name = Optional(str)
+    display_name = Optional(str, sql_default="''")
     birthdate = Optional(date)
     token = Required(Json, volatile=True)
     spotify_premium = Required(bool)
-    api_calls = Required(int, default=0, volatile=True)
-    created_at = Required(datetime, default=datetime.utcnow)
-    last_usage_at = Required(datetime, default=datetime.utcnow, volatile=True)
+    api_calls = Required(int, default=0, sql_default="0", volatile=True)
+    created_at = Required(
+        datetime, default=datetime.utcnow, sql_default=SQL_DEFAULT.now
+    )
+    last_usage_at = Required(
+        datetime, default=datetime.utcnow, sql_default=SQL_DEFAULT.now, volatile=True
+    )
     images = Set("Image", cascade_delete=True)
     top_artists = Set("Artist")
     disliked_artists = Set("Artist")
@@ -170,7 +342,7 @@ class User(db.Entity, ImageMixin):
     disliked_countries = Set("Country")
     top_cities = Set("City")
     disliked_cities = Set("City")
-    top_expires_at = Optional(Json, volatile=True)
+    top_expires_at = Required(Json, volatile=True, default=dict, sql_default="'{}'")
 
     def to_dict(self, *args, **kwargs):  # pylint: disable=arguments-differ
         _dict = super().to_dict(*args, **kwargs)
@@ -199,18 +371,17 @@ class User(db.Entity, ImageMixin):
             for image in user.images:
                 image.color = color
         images = [Image.get(url=image.url) or Image(**image) for image in user.images]
-        spotify_user = (
-            SpotifyUser.get(id=user.id)
-            or SpotifyUser(id=user.id, name=user.get("display_name") or "")
+        spotify_user = SpotifyUser.get(id=user.id) or SpotifyUser(
+            id=user.id, name=user.get("display_name") or ""
         )
         return cls(
             id=user.user_id,
             spotify_premium=user.product == "premium",
             spotify_user=spotify_user,
             username=user.id,
-            email=user.email,
+            email=user.email or str(user.user_id),
             token=user.token,
-            country=Country.from_str(code=user.country),
+            country=Country.from_str(user.country),
             images=images,
             display_name=user.display_name or "",
             birthdate=datetime.strptime(user.birthdate, "%Y-%m-%d")
@@ -297,6 +468,27 @@ class User(db.Entity, ImageMixin):
                 self.top_cities.add(city)
                 self.disliked_cities.remove(city)
 
+    @classmethod
+    async def like_pg(cls, conn, user_id, **kwargs):
+        for key, value in kwargs.items():
+            await conn.execute(SQL.like.format(key), user_id, value)
+
+    @classmethod
+    async def dislike_pg(cls, conn, spotify, **kwargs):
+        artist = kwargs.pop("artist", None)
+        if artist:
+            artist = await spotify.artist(artist)
+            await conn.execute(
+                SQL.dislike_artist.format("artist"),
+                spotify.user_id,
+                artist.id,
+                artist.name,
+                artist.followers.total or 0,
+                artist.popularity or 0,
+            )
+        for key, value in kwargs.items():
+            await conn.execute(SQL.dislike.format(key), spotify.user_id, value)
+
     def top_expired(self, time_range):
         time_range = TimeRange(time_range).value
         return (
@@ -316,13 +508,12 @@ class User(db.Entity, ImageMixin):
                 username=cls.DEFAULT_USERNAME,
                 email=cls.DEFAULT_EMAIL,
                 token={},
-                country=Country.from_str(code="US"),
+                country=Country.from_str("US"),
                 spotify_premium=False,
             )
 
     @staticmethod
     def token_updater(_id):
-
         @db_session
         def update(token):
             User[_id].token = token
@@ -338,30 +529,33 @@ class Image(db.Entity):
     url = PrimaryKey(str)
     height = Optional(int)
     width = Optional(int)
-    color = Optional(str)
+    color = Optional(str, sql_default="''")
     playlist = Optional("Playlist")
     artist = Optional("Artist")
     genre = Optional("Genre")
     country = Optional("Country")
     city = Optional("City")
     user = Optional("User")
-    unsplash_id = Optional(str, index=True)
-    unsplash_user_fullname = Optional(str)
-    unsplash_user_username = Optional(str)
+    unsplash_id = Optional(str, index=True, sql_default="''")
+    unsplash_user_fullname = Optional(str, sql_default="''")
+    unsplash_user_username = Optional(str, sql_default="''")
 
     # pylint: disable=no-self-use
 
-    def unsplash_url(self):
+    @classmethod
+    def unsplash_url(cls):
         return f"https://unsplash.com/?utm_source={config.unsplash.app_name}&utm_medium=referral"
 
-    def unsplash_user_url(self):
-        return f"https://unsplash.com/@{self.unsplash_user_username}?utm_source={config.unsplash.app_name}&utm_medium=referral"
+    @classmethod
+    def unsplash_user_url(cls, username):
+        return f"https://unsplash.com/@{username}?utm_source={config.unsplash.app_name}&utm_medium=referral"
 
-    def unsplash_credits(self):
+    @classmethod
+    def unsplash_credits(cls, user_fullname, username):
         return {
-            "user_name": self.unsplash_user_fullname,
-            "user_url": self.unsplash_user_url(),
-            "site_url": self.unsplash_url(),
+            "user_name": user_fullname,
+            "user_url": cls.unsplash_user_url(username),
+            "site_url": cls.unsplash_url(),
         }
 
     @staticmethod
@@ -379,6 +573,20 @@ class Image(db.Entity):
         color = ColorThief(image_file).get_color(quality=1)
         return f"#{color[0]:02x}{color[1]:02x}{color[2]:02x}"
 
+    @classmethod
+    async def download_pg(cls, conn, url):
+        unsplash_id = await conn.fetchval(
+            "SELECT unsplash_id FROM images WHERE url = $1", url
+        )
+        image = None
+        async with aiohttp.ClientSession() as client:
+            async with client.get(url) as resp:
+                _, image = await asyncio.gather(
+                    Unsplash.photo.download(unsplash_id, without_content=True),
+                    resp.read(),
+                )
+        return image
+
     async def download(self):
         image = None
         async with aiohttp.ClientSession() as client:
@@ -393,7 +601,7 @@ class Image(db.Entity):
 class SpotifyUser(db.Entity):
     _table_ = "spotify_users"
     id = PrimaryKey(str)  # pylint: disable=redefined-builtin
-    name = Optional(str)
+    name = Optional(str, sql_default="''")
     user = Optional("User")
     playlists = Set("Playlist")
 
@@ -446,22 +654,17 @@ class Country(db.Entity, ImageMixin):
             return False
 
     @classmethod
-    def from_str(cls, name=None, code=None):
-        iso_country = None
-        if name == "UK":
+    def get_iso_country(cls, country):
+        iso_country, code, name = None, None, None
+        if len(country) == 2:
+            code = country
+        else:
+            name = country
+
+        if country == "UK":
             iso_country = countries.get(alpha_2="GB")
-        elif name == "USA":
+        elif country == "USA":
             iso_country = countries.get(alpha_2="US")
-        elif name is not None:
-            try:
-                iso_country = countries.get(name=name)
-            except KeyError:
-                try:
-                    iso_country = countries.get(official_name=name)
-                except KeyError:
-                    iso_country = first(
-                        countries, key=lambda c: cls._name_match(c, name.lower())
-                    )
         elif code is not None:
             try:
                 iso_country = countries.get(alpha_2=code)
@@ -469,8 +672,30 @@ class Country(db.Entity, ImageMixin):
                 iso_country = first(
                     countries, key=lambda c: code.lower() in c.alpha_2.lower()
                 )
+        elif name is not None:
+            try:
+                iso_country = countries.get(name=name)
+            except KeyError:
+                try:
+                    iso_country = countries.get(official_name=name)
+                except KeyError:
+                    try:
+                        iso_country = countries.get(common_name=name)
+                    except KeyError:
+                        iso_country = first(
+                            countries, key=lambda c: cls._name_match(c, name.lower())
+                        )
+        if not iso_country:
+            logger.error(
+                "Could not find a country with name=%s and code=%s", name, code
+            )
+            return None
+        return iso_country
+
+    @classmethod
+    def from_str(cls, country):
+        iso_country = cls.get_iso_country(country)
         if not iso_country:
-            logger.error(f"Could not find a country with name={name} and code={code}")
             return None
 
         return cls.get(name=iso_country.name) or cls(
@@ -505,7 +730,7 @@ class Playlist(db.Entity, ImageMixin):
     YEAR = "(?P<year>[0-9]{4})"
     GENRE = "(?P<genre>.+)"
     CITY = "(?P<city>.+)"
-    COUNTRY = "(?P<country>.+)"
+    COUNTRY = "(?P<country>.+?)"
     COUNTRY_CODE = "(?P<country_code>[A-Z]{2})"
     DATE = "(?P<date>[0-9]{8})"
     INTRO_POPULARITY = "(?P<popularity>Intro)"
@@ -515,14 +740,13 @@ class Playlist(db.Entity, ImageMixin):
     PATTERNS = OrderedDict(
         intro_to_genre=re.compile(f"^{INTRO_POPULARITY} to {GENRE}$"),
         sound_of_city=re.compile(f"^The Sound of {CITY} {COUNTRY_CODE}$"),
-        needle=re.compile(
-            f"^The Needle / {COUNTRY} {DATE}(?: - {NEEDLE_POPULARITY})?$"
-        ),
+        needle=re.compile(f"^The Needle / {COUNTRY}(?: - {NEEDLE_POPULARITY})?$"),
         pine_needle=re.compile(f"^The Pine Needle / {COUNTRY}$"),
         year_in_genre=re.compile(f"^{YEAR} in {GENRE}$"),
         meta_genre=re.compile(f"^Meta{GENRE_POPULARITY_LOWER}: {GENRE}$"),
         meta_year_in_genre=re.compile(f"^Meta{YEAR}: {GENRE}$"),
         sound_of_genre=re.compile(f"^The {GENRE_POPULARITY_TITLE} of {GENRE}$"),
+        women_filter_genre=re.compile(f"^A ♀Filter for {GENRE}$"),
     )
 
     class Popularity(IntEnum):
@@ -539,19 +763,20 @@ class Playlist(db.Entity, ImageMixin):
     id = PrimaryKey(str)  # pylint: disable=redefined-builtin
     collaborative = Required(bool)
     name = Required(str)
-    description = Optional(str)
+    description = Optional(str, sql_default="''")
     owner = Required(SpotifyUser)
     public = Required(bool)
     snapshot_id = Required(str)
     tracks = Required(int)
-    popularity = Optional(int, index=True)
+    popularity = Optional(int, index=True, sql_default="0")
     genre = Optional(Genre)
     country = Optional(Country)
     city = Optional(City)
     date = Optional(date, index=True)
     year = Optional(int)
-    christmas = Optional(bool, index=True)
-    meta = Optional(bool, index=True)
+    women = Optional(bool, index=True, sql_default=SQL_DEFAULT.bool_false)
+    christmas = Optional(bool, index=True, sql_default=SQL_DEFAULT.bool_false)
+    meta = Optional(bool, index=True, sql_default=SQL_DEFAULT.bool_false)
     images = Set(Image, cascade_delete=True)
 
     def play(self, client, device=None):
@@ -573,29 +798,22 @@ class Playlist(db.Entity, ImageMixin):
     def get_fields(cls, groups):
         fields = {}
         if "year" in groups:
-            year = groups["year"]
-            fields["year"] = int(year)
-            fields["date"] = datetime(int(year), 1, 1)
+            fields["year"] = int(groups["year"])
+            fields["date"] = date(fields["year"], 1, 1)
             fields["popularity"] = cls.Popularity.YEAR.value
         if "city" in groups and "country_code" in groups:
-            city = groups["city"]
-            country_code = groups["country_code"]
-            fields["country"] = Country.from_str(code=country_code)
-            fields["city"] = City.get(name=city) or City(
-                name=city, country=fields["country"]
-            )
+            fields["country"] = groups["country_code"]
+            fields["city"] = groups["city"]
             fields["popularity"] = cls.Popularity.SOUND.value
         if "genre" in groups:
             genre = groups["genre"].lower()
-            fields["genre"] = Genre.get(name=genre) or Genre(name=genre)
+            fields["genre"] = genre
             if "christmas" in genre:
                 fields["christmas"] = True
         if "country" in groups:
-            country = groups["country"]
-            fields["country"] = Country.from_str(name=country)
-        if "date" in groups:
-            _date = groups["date"]
-            fields["date"] = datetime.strptime(_date, "%Y%m%d").date()
+            fields["country"] = groups["country"]
+        if groups.get("date"):
+            fields["date"] = datetime.strptime(groups["date"], "%Y%m%d").date()
         if "popularity" in groups:
             popularity = groups["popularity"]
             if popularity:
@@ -605,14 +823,37 @@ class Playlist(db.Entity, ImageMixin):
         return fields
 
     @classmethod
-    def from_dict(
-        cls, playlist
-    ):  # pylint: disable=too-many-return-statements,too-many-statements
-        owner = (
-            SpotifyUser.get(id=playlist.owner.id)
-            or SpotifyUser(
-                id=playlist.owner.id, name=playlist.owner.get("display_name") or ""
-            )
+    def from_dict_pg(cls, playlist):
+        fields = {
+            "id": playlist.id,
+            "description": playlist.description or "",
+            "collaborative": playlist.collaborative,
+            "name": playlist.name,
+            "owner": playlist.owner.id,
+            "public": playlist.public,
+            "snapshot_id": playlist.snapshot_id,
+            "tracks": playlist.tracks.total,
+            "women": playlist.name.startswith("A ♀Filter for"),
+            "christmas": (
+                "Pine Needle" in playlist.name or "christmas" in playlist.name.lower()
+            ),
+            "meta": playlist.name.startswith("Meta"),
+        }
+        for pattern in cls.PATTERNS.values():
+            match = pattern.match(playlist.name)
+            if match:
+                groups = match.groupdict()
+                fields.update(cls.get_fields(groups))
+                break
+
+        else:
+            logger.warning("No pattern matches the playlist: %s", playlist.name)
+        return fields
+
+    @classmethod
+    def from_dict(cls, playlist):
+        owner = SpotifyUser.get(id=playlist.owner.id) or SpotifyUser(
+            id=playlist.owner.id, name=playlist.owner.get("display_name") or ""
         )
         fields = {
             "id": playlist.id,
@@ -622,8 +863,9 @@ class Playlist(db.Entity, ImageMixin):
             "public": playlist.public,
             "snapshot_id": playlist.snapshot_id,
             "tracks": playlist.tracks.total,
-            "christmas": "Pine Needle" in playlist.name
-            or "christmas" in playlist.name.lower(),
+            "christmas": (
+                "Pine Needle" in playlist.name or "christmas" in playlist.name.lower()
+            ),
             "meta": playlist.name.startswith("Meta"),
             "images": [Image.get(url=im.url) or Image(**im) for im in playlist.images],
         }
@@ -635,7 +877,18 @@ class Playlist(db.Entity, ImageMixin):
                 break
 
         else:
-            logger.warning(f"No pattern matches the playlist: {playlist.name}")
+            logger.warning("No pattern matches the playlist: %s", playlist.name)
+
+        if fields.get("country"):
+            fields["country"] = Country.from_str(fields["country"])
+        if fields.get("city"):
+            fields["city"] = City.get(name=fields["city"]) or City(
+                name=fields["city"], country=fields["country"]
+            )
+        if fields.get("genre"):
+            fields["genre"] = Genre.get(name=fields["genre"]) or Genre(
+                name=fields["genre"]
+            )
         return cls(**fields)
 
 
@@ -647,7 +900,7 @@ class Artist(db.Entity, ImageMixin):
     genres = Set(Genre)
     fans = Set(User, reverse="top_artists", table="artist_fans")
     haters = Set(User, reverse="disliked_artists", table="artist_haters")
-    popularity = Optional(int)
+    popularity = Optional(int, sql_default="0")
     images = Set(Image, cascade_delete=True)
 
     def play(self, client, device=None):
@@ -751,8 +1004,7 @@ if config.database.connection.filename:
 db.bind(**config.database.connection)
 
 GENERATE_MAPPING = os.getenv("SPFY_GENERATE_MAPPING")
-if (
-    config.database.generate_mapping
-    and (GENERATE_MAPPING is None or GENERATE_MAPPING == "true")
+if config.database.generate_mapping and (
+    GENERATE_MAPPING is None or GENERATE_MAPPING == "true"
 ):
     db.generate_mapping(create_tables=True)
