@@ -1,7 +1,8 @@
 import logging
+from inspect import isawaitable
 
 import aiohttp
-from oauthlib.common import urldecode, generate_token
+from oauthlib.common import generate_token, urldecode
 from oauthlib.oauth2.rfc6749.errors import TokenExpiredError
 
 from oauthlib.oauth2 import (  # isort:skip
@@ -14,7 +15,6 @@ log = logging.getLogger(__name__)
 
 
 class TokenUpdated(Warning):
-
     def __init__(self, token):
         super(TokenUpdated, self).__init__()
         self.token = token
@@ -100,15 +100,6 @@ class OAuth2Session(aiohttp.ClientSession):
             "protected_request": set(),
         }
 
-    # pylint: disable=arguments-differ
-
-    def __del__(self, *args, **kwargs):
-        if not self.closed:
-            if self._connector_owner:
-                self._connector.close()
-            self._connector = None
-        super().__del__(*args, **kwargs)
-
     def new_state(self):
         """Generates a state string to be used in authorizations."""
         try:
@@ -173,9 +164,16 @@ class OAuth2Session(aiohttp.ClientSession):
         :return: authorization_url, state
         """
         state = state or self.new_state()
-        return self._client.prepare_request_uri(
-            url, redirect_uri=self.redirect_uri, scope=self.scope, state=state, **kwargs
-        ), state
+        return (
+            self._client.prepare_request_uri(
+                url,
+                redirect_uri=self.redirect_uri,
+                scope=self.scope,
+                state=state,
+                **kwargs,
+            ),
+            state,
+        )
 
     #  pylint: disable=too-many-locals
 
@@ -442,7 +440,9 @@ class OAuth2Session(aiohttp.ClientSession):
                         log.debug(
                             "Updating token to %s using %s.", token, self.token_updater
                         )
-                        self.token_updater(token)
+                        res = self.token_updater(token)
+                        if isawaitable(res):
+                            await res
                         url, headers, data = self._client.add_token(
                             url, http_method=method, body=data, headers=headers
                         )
