@@ -1,19 +1,19 @@
-import uuid
 import socket
 import threading
+import uuid
 from pathlib import Path
 from wsgiref.simple_server import make_server
 
 import hug
-from pony.orm import get
 from cachecontrol import CacheControlAdapter
-from oauthlib.oauth2 import BackendApplicationClient
-from requests_oauthlib import OAuth2Session
 from cachecontrol.caches.file_cache import FileCache
+from oauthlib.oauth2 import BackendApplicationClient
+from pony.orm import get
+from requests_oauthlib import OAuth2Session
 
-from .. import root, config, logger
-from ..cache import User, select, db_session
-from ..constants import API, AuthFlow, AllScopes
+from .. import config, logger, root
+from ..cache import User, db_session, select
+from ..constants import API, AllScopes, AuthFlow
 from ..exceptions import SpotifyCredentialsException
 
 AUTH_HTML_FILE = root / "html" / "auth_message.html"
@@ -21,7 +21,6 @@ CACHE_FILE = Path.home() / ".cache" / "spfy" / ".web_cache"
 
 
 class AuthMixin:
-
     def __init__(
         self,
         *args,
@@ -29,15 +28,18 @@ class AuthMixin:
         client_secret=None,
         redirect_uri=None,
         user_id=None,
+        username=None,
         **kwargs,
     ):
         super().__init__(*args, **kwargs)
         self.client_id = client_id or config.app.client_id
         self.client_secret = client_secret or config.app.client_secret
         self.redirect_uri = self._get_redirect_uri(redirect_uri)
-        self.session = None
         self.user_id = user_id
+        self.username = username
         self.callback_reached = threading.Event()
+        self.flow = None
+        self.session = None
 
     @staticmethod
     def _get_redirect_uri(redirect_uri):
@@ -81,6 +83,7 @@ class AuthMixin:
         auth_response=None,
         scope=AllScopes,
     ):
+        self.flow = AuthFlow.AUTHORIZATION_CODE
         session = self.session or self.get_session(
             self.client_id,
             redirect_uri=self.redirect_uri,
@@ -98,6 +101,7 @@ class AuthMixin:
             user = get(u for u in User if u.username == username or u.email == email)
             if user:
                 self.user_id = user.id
+                self.username = user.username
                 session.token = user.token
                 session.token_updater = User.token_updater(user.id)
                 return session
@@ -112,15 +116,20 @@ class AuthMixin:
                 authorization_response=auth_response,
             )
             user_details = self.current_user()
-            user = select(
-                u
-                for u in User
-                if u.username == user_details.id or u.email == user_details.email
-            ).for_update().get()
+            user = (
+                select(
+                    u
+                    for u in User
+                    if u.username == user_details.id or u.email == user_details.email
+                )
+                .for_update()
+                .get()
+            )
             if user:
                 user.token = token
                 if user.id != self.user_id:
                     self.user_id = user.id
+                    self.username = user.username
             elif self.user_id:
                 user = User.get_for_update(id=self.user_id)
                 if user:
@@ -130,14 +139,17 @@ class AuthMixin:
                 user_details["user_id"] = self.user_id
                 user_details["token"] = token
                 user = User.from_dict(user_details)
+                self.username = user.username
             return session
 
         return session
 
     @db_session
     def authenticate_server(self):
+        self.flow = AuthFlow.CLIENT_CREDENTIALS
         default_user = User.default()
         self.user_id = default_user.id
+        self.username = default_user.username
         session = self.session or self.get_session(
             client=BackendApplicationClient(self.client_id)
         )
@@ -156,10 +168,10 @@ class AuthMixin:
         if not (self.client_id and self.client_secret):
             raise SpotifyCredentialsException
 
-        flow = AuthFlow(flow)
-        if flow == AuthFlow.CLIENT_CREDENTIALS:
+        self.flow = AuthFlow(flow)
+        if self.flow == AuthFlow.CLIENT_CREDENTIALS:
             self.session = self.authenticate_server()
-        elif flow == AuthFlow.AUTHORIZATION_CODE:
+        elif self.flow == AuthFlow.AUTHORIZATION_CODE:
             self.session = self.authenticate_user(**auth_params)
             if not self.session.token:
                 if config.auth.callback.enabled:
