@@ -1,6 +1,6 @@
 from mailer import Mailer, Message
 
-from .. import root, config, logger
+from .. import config, logger, root
 
 LOGIN_HTML = root / "html" / "login.html"
 
@@ -9,13 +9,12 @@ LOGIN_HTML = root / "html" / "login.html"
 
 
 class EmailMixin:
-
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
     @staticmethod
     def send_auth_email(email, auth_url):
-        logger.info(f"Login here to use Spotify API: {auth_url}")
+        logger.info("Login here to use Spotify API: %s", auth_url)
         mailer = Mailer(**config.email)
         # pylint: disable=no-member
         html_content = LOGIN_HTML.read_text().replace(
