@@ -18,7 +18,6 @@ def cap(volume, _min=1, _max=100):
 
 
 class VolumeControl(abc.ABC):
-
     @abc.abstractmethod
     def unmute(self):
         pass
@@ -36,8 +35,8 @@ class VolumeControl(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def should_stop_fading(self, device_volume, old_volume):
-        return abs(device_volume - old_volume) > 2
+    def should_stop_fading(self, device_volume, old_volume, step):
+        return abs(device_volume - old_volume) > (step + 2)
 
     def fade(
         self, limit=100, start=1, step=1, seconds=VOLUME_FADE_SECONDS, force=False
@@ -51,22 +50,25 @@ class VolumeControl(abc.ABC):
                 sleep(delay)
                 device_volume = self.volume
                 old_volume = next_volume - step
-                if not force and self.should_stop_fading(device_volume, old_volume):
+                if not force and self.should_stop_fading(
+                    device_volume, old_volume, step
+                ):
                     logger.debug(
-                        f"""A stop fading condition was met:
-                        Current volume: {device_volume}
-                        Old volume: {old_volume}"""
+                        """A stop fading condition was met:
+                        Current volume: %s
+                        Old volume: %s""",
+                        device_volume,
+                        old_volume,
                     )
                     break
 
-                logger.debug(f"Setting volume to {next_volume}")
+                logger.debug("Setting volume to %s", next_volume)
                 self.volume = next_volume
         except Exception as exc:
             logger.exception(exc)
 
 
 class SpotifyVolumeControl(VolumeControl):
-
     def __init__(self, client, device=None):
         self.spotify = client
         self.device = device
@@ -82,10 +84,13 @@ class SpotifyVolumeControl(VolumeControl):
         else:
             self.volume = 1
 
-    def should_stop_fading(self, device_volume, old_volume):
+    def should_stop_fading(self, device_volume, old_volume, step):
         is_playing = self.spotify.current_playback().is_playing
-        logger.debug(f"Spotify playing: {is_playing}")
-        return super().should_stop_fading(device_volume, old_volume) or not is_playing
+        logger.debug("Spotify playing: %s", is_playing)
+        return (
+            super().should_stop_fading(device_volume, old_volume, step)
+            or not is_playing
+        )
 
     @property
     def volume(self):
@@ -97,7 +102,6 @@ class SpotifyVolumeControl(VolumeControl):
 
 
 class SpotifyVolumeControlAsync(VolumeControl):
-
     def __init__(self, client, device=None):
         self.spotify = client
         self.device = device
@@ -117,11 +121,15 @@ class SpotifyVolumeControlAsync(VolumeControl):
         playback = await self.spotify.current_playback()
         return playback and playback.is_playing
 
-    async def should_stop_fading(self, device_volume, old_volume):
+    async def should_stop_fading(self, device_volume, old_volume, step):
         is_playing = await self.is_playing()
-        logger.debug(f"Spotify playing: {is_playing}")
-        return super().should_stop_fading(device_volume, old_volume) or not is_playing
+        logger.debug("Spotify playing: %s", is_playing)
+        return (
+            super().should_stop_fading(device_volume, old_volume, step)
+            or not is_playing
+        )
 
+    @property
     async def volume(self):  # pylint: disable=method-hidden
         device = await self.spotify.get_device(device=self.device)
         return int(device.volume_percent or 0)
@@ -130,19 +138,18 @@ class SpotifyVolumeControlAsync(VolumeControl):
         await self.spotify.volume(cap(val), device=self.device)
 
     async def force_fade(self, limit, step, delay):
-
         def fade_done(vol):
             if step > 0:
                 return vol >= limit
             return vol <= limit
 
-        device_volume = await self.volume()
+        device_volume = await self.volume
         while not fade_done(device_volume) and (await self.is_playing()):
             next_volume = device_volume + step
-            logger.debug(f"Setting volume to {next_volume}")
+            logger.debug("Setting volume to %s", next_volume)
             await self.set_volume(next_volume)
             await asyncio.sleep(delay)
-            device_volume = await self.volume()
+            device_volume = await self.volume
 
     async def fade(
         self, limit=100, start=1, step=1, seconds=VOLUME_FADE_SECONDS, force=False
@@ -154,32 +161,33 @@ class SpotifyVolumeControlAsync(VolumeControl):
         if force:
             await self.force_fade(limit, step, delay)
         else:
-            device_volume = await self.volume()
+            device_volume = await self.volume
             for next_volume in range(start + step, limit + step, step):
                 await asyncio.sleep(delay)
-                device_volume = await self.volume()
+                device_volume = await self.volume
                 old_volume = next_volume - step
                 should_stop_fading = await self.should_stop_fading(
-                    device_volume, old_volume
+                    device_volume, old_volume, step
                 )
                 if should_stop_fading:
                     logger.debug(
-                        f"""A stop fading condition was met:
-                        Current volume: {device_volume}
-                        Old volume: {old_volume}"""
+                        """A stop fading condition was met:
+                        Current volume: %s
+                        Old volume: %s""",
+                        device_volume,
+                        old_volume,
                     )
                     break
 
-                logger.debug(f"Setting volume to {next_volume}")
+                logger.debug("Setting volume to %s", next_volume)
                 await self.set_volume(next_volume)
 
-        device_volume = await self.volume()
-        if device_volume <= 1:
+        device_volume = await self.volume
+        if device_volume <= abs(step):
             await self.spotify.pause_playback()
 
 
 class AlsaVolumeControl(VolumeControl):
-
     def __init__(self, mixer_name, device=None):
         if isinstance(device, str):
             kwargs = dict(device=device)
@@ -197,10 +205,10 @@ class AlsaVolumeControl(VolumeControl):
     def unmute(self):
         self.mixer.setmute(0)
 
-    def should_stop_fading(self, device_volume, old_volume):
+    def should_stop_fading(self, device_volume, old_volume, step):
         is_mute = self.mixer.getmute()[0]
-        logger.debug(f"Mute status: {is_mute}")
-        return super().should_stop_fading(device_volume, old_volume) or is_mute
+        logger.debug("Mute status: %s", is_mute)
+        return super().should_stop_fading(device_volume, old_volume, step) or is_mute
 
     @property
     def volume(self):
@@ -213,7 +221,6 @@ class AlsaVolumeControl(VolumeControl):
 
 
 class ApplescriptVolumeControl(VolumeControl):
-
     def __init__(self, device=None):
         self.device = device
         self.old_volume = None
@@ -235,15 +242,19 @@ class ApplescriptVolumeControl(VolumeControl):
 
     @staticmethod
     def switch_audio_device(device):
-        logger.debug(f"Switching audio device: {device}")
+        logger.debug("Switching audio device: %s", device)
         subprocess.call(f'/usr/local/bin/SwitchAudioSource -s "{device}"', shell=True)
 
-    def should_stop_fading(self, device_volume, old_volume):
-        is_playing = self.osascript(
-            'tell application "Spotify" to get player state'
-        ) == "playing"
-        logger.debug(f"Spotify playing: {is_playing}")
-        return super().should_stop_fading(device_volume, old_volume) or not is_playing
+    def should_stop_fading(self, device_volume, old_volume, step):
+        is_playing = (
+            self.osascript('tell application "Spotify" to get player state')
+            == "playing"
+        )
+        logger.debug("Spotify playing: %s", is_playing)
+        return (
+            super().should_stop_fading(device_volume, old_volume, step)
+            or not is_playing
+        )
 
     def spotify_mute(self):
         self.old_volume = self.spotify_volume
@@ -291,7 +302,6 @@ class ApplescriptVolumeControl(VolumeControl):
 
 
 class LinuxVolumeControl(AlsaVolumeControl):
-
     def __init__(
         self, spotify_client, alsa_mixer_name, spotify_device=None, alsa_device=None
     ):
@@ -300,18 +310,18 @@ class LinuxVolumeControl(AlsaVolumeControl):
             spotify_client, spotify_device
         )
 
-    def should_stop_fading(self, device_volume, old_volume):
-        return (
-            super().should_stop_fading(device_volume, old_volume)
-            or self.spotify_should_stop_fading(device_volume, old_volume)
-        )
+    def should_stop_fading(self, device_volume, old_volume, step):
+        return super().should_stop_fading(
+            device_volume, old_volume, step
+        ) or self.spotify_should_stop_fading(device_volume, old_volume, step)
 
-    def spotify_should_stop_fading(self, device_volume, old_volume):
-        return self.spotify_volume_control.should_stop_fading(device_volume, old_volume)
+    def spotify_should_stop_fading(self, device_volume, old_volume, step):
+        return self.spotify_volume_control.should_stop_fading(
+            device_volume, old_volume, step
+        )
 
 
 class LinuxVolumeControlAsync(AlsaVolumeControl):
-
     def __init__(
         self, spotify_client, alsa_mixer_name, spotify_device=None, alsa_device=None
     ):
@@ -320,15 +330,14 @@ class LinuxVolumeControlAsync(AlsaVolumeControl):
             spotify_client, spotify_device
         )
 
-    async def should_stop_fading(self, device_volume, old_volume):
-        return (
-            super().should_stop_fading(device_volume, old_volume)
-            or await self.spotify_should_stop_fading(device_volume, old_volume)
-        )
+    async def should_stop_fading(self, device_volume, old_volume, step):
+        return super().should_stop_fading(
+            device_volume, old_volume, step
+        ) or await self.spotify_should_stop_fading(device_volume, old_volume, step)
 
-    async def spotify_should_stop_fading(self, device_volume, old_volume):
+    async def spotify_should_stop_fading(self, device_volume, old_volume, step):
         return await self.spotify_volume_control.should_stop_fading(
-            device_volume, old_volume
+            device_volume, old_volume, step
         )
 
     async def fade(
@@ -344,17 +353,19 @@ class LinuxVolumeControlAsync(AlsaVolumeControl):
                 device_volume = self.volume
                 old_volume = next_volume - step
                 should_stop_fading = await self.should_stop_fading(
-                    device_volume, old_volume
+                    device_volume, old_volume, step
                 )
                 if not force and should_stop_fading:
                     logger.debug(
-                        f"""A stop fading condition was met:
-                        Current volume: {device_volume}
-                        Old volume: {old_volume}"""
+                        """A stop fading condition was met:
+                        Current volume: %s
+                        Old volume: %s""",
+                        device_volume,
+                        old_volume,
                     )
                     break
 
-                logger.debug(f"Setting volume to {next_volume}")
+                logger.debug("Setting volume to %s", next_volume)
                 self.volume = next_volume
         except Exception as exc:
             logger.exception(exc)
